import { errorHandler } from '@gtpl/shared-utils/error-messages';
import { Body, Controller, Get, Param, Post, Query, Req, Res, UploadedFiles, UseInterceptors } from '@nestjs/common';
import { FilesInterceptor } from '@nestjs/platform-express';
import { ApiConsumes, ApiHeader, ApiParam, ApiQuery, ApiTags } from '@nestjs/swagger';
import { extname } from 'path';
import { diskStorage } from 'multer';
import { imageFileFilter } from '../shared/file-upload';
import { DeleteIdeaDto, IdeaDTO, LikeIdeaDto } from './dto/idea.dto';
import { AllIdeaResponseModel, IdeaResponseModel, SignalIdeaResponseModel } from './idea-response-module';
import { IdeaService } from './idea.service';
@ApiTags('Idea')
@Controller('idea')
export class IdeaController {
    constructor (
        private ideaService: IdeaService,
    ){}

    /**
     * create text/url post 
     * @param queryParam which returns api version
     * @param postDto which passes details to be saved
     * @param request which returns headers 
     * @returns postResponse which returns the updated data along with status (true/false), errortype, intlcode, internal message which provides message the calling function.   
     */
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/createIdea') 
    @ApiQuery({name:'apiVersion'})   
    @UseInterceptors(
		FilesInterceptor('attachments', 5, {
			fileFilter: imageFileFilter,
			storage: diskStorage({
				destination: './public/ideaImages',
				filename: function (req, file, callback) {
					const name = file.originalname.split('.')[0];
					const fileExtName = extname(file.originalname);
					const randomName = Array(4)
						.fill(null)
						.map(() => Math.round(Math.random() * 16).toString(16))
						.join('');
					callback(null, `${name}-${randomName}${fileExtName}`);
				}
			}),			
		}),
    )
    @ApiConsumes('multipart/form-data')
    async createIdea(@Query() queryParam,@UploadedFiles() attachments,@Body() iDTO:IdeaDTO,@Req() request:Request): Promise<IdeaResponseModel> {
        try {
            const saveData = await this.ideaService.createIdea(queryParam, iDTO);
            const response = [];
            attachments.forEach(file => {
                const fileReponse = {                   
                    ideaId:saveData.Id,
                    attachmentsUrl: `idea/ideaImages/${file.filename}`
                };
                response.push(fileReponse);
            })
            return await this.ideaService.createIdeaFiles(queryParam, response,saveData.Id);
        } catch (error) {
            return errorHandler(IdeaResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/updateIdea') 
    @ApiQuery({name:'apiVersion'})   
    @UseInterceptors(
		FilesInterceptor('attachments', 5, {
			fileFilter: imageFileFilter,
			storage: diskStorage({
				destination: './public/ideaImages',
				filename: function (req, file, callback) {
					const name = file.originalname.split('.')[0];
					const fileExtName = extname(file.originalname);
					const randomName = Array(4)
						.fill(null)
						.map(() => Math.round(Math.random() * 16).toString(16))
						.join('');
					callback(null, `${name}-${randomName}${fileExtName}`);
				}
			}),			
		}),
    )
    @ApiConsumes('multipart/form-data')
    async updateIdea(@Query() queryParam,@UploadedFiles() attachments,@Body() iDto:IdeaDTO,@Req() request:Request): Promise<IdeaResponseModel> {
        try {
            const response = [];
            attachments.forEach(file => {
                const fileReponse = {                   
                    ideaId:iDto.Id,
                    attachmentsUrl: `idea/ideaImages/${file.filename}`,
                };
                response.push(fileReponse);
            })
            return await this.ideaService.updateIdea(queryParam, response,iDto);
        } catch (error) {
            return errorHandler(IdeaResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Get('/getAllIdeas/:userId')
    @ApiQuery({name:'apiVersion'})
    @ApiQuery({name:'skip'}) 
    @ApiParam({name:'userId'})   
    async getAllIdeas(@Query() queryParams, @Param('userId') userId, @Req() request:Request):Promise<AllIdeaResponseModel> {
        try {           
            return await this.ideaService.getAllIdeas(queryParams,userId,request);
        } catch (error) {
            return errorHandler(AllIdeaResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Get('/getIdeaById/:Id')
    @ApiQuery({name:'apiVersion'})
    @ApiParam({name:'Id'})
    async getIdeaById(@Query() queryParams,@Param('Id') Id, @Req() request:Request):Promise<SignalIdeaResponseModel> {
        try {
            return await this.ideaService.getIdeaById(queryParams,Id,request);
        } catch (error) {
            return errorHandler(SignalIdeaResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/likeIdea')
    @ApiQuery({name:'apiVersion'})
    async likeIdea(@Query() queryParams, @Body() lDto: LikeIdeaDto, @Req() request: Request): Promise<IdeaResponseModel> {
        try {   
            return await this.ideaService.likeIdea(queryParams,lDto, request);
        } catch (error) {
            return errorHandler(IdeaResponseModel, error);
        }
    }

    @Get('ideaImages/:imgpath')
    @ApiParam({name:'imgpath'})
    seeUploadedFile(@Param('imgpath') attachments, @Res() res) {
		try {
            return res.sendFile(attachments, { root: './public/ideaImages' });
		} catch (error) {
			return error.message;
		}
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/deleteIdea')
    @ApiQuery({name:'apiVersion'})
    async deleteIdea(@Query() queryParams, @Body() delDto: DeleteIdeaDto, @Req() request: Request): Promise<IdeaResponseModel> {
        try {   
            return await this.ideaService.deleteIdea(queryParams,delDto, request);
        } catch (error) {
            return errorHandler(IdeaResponseModel, error);
        }
    }   
}
