import { KafkaEventBusService } from "./kafka-event-bus.service";
import EventBusOptions from "./dto/event-bus-options";
import Event from "./dto/event";
import { KafkaTransportMock } from "./transport/mocks/kafka-transport-mock";
import LoggerMock from "./transport/mocks/logger-mock";
import SpyInstance = jest.SpyInstance;


describe("KafkaEventBusService", () => {
    let service: KafkaEventBusService;

    const options: EventBusOptions = {
        brokers: ["localhost:9092"],
        clientId: "something",
        groupId: "testgroup",
        topics: ["test topic", "test topic 2"],
    }
    let transport: KafkaTransportMock = new KafkaTransportMock(options);

    beforeEach(() => {
        jest.clearAllMocks();
        service = new KafkaEventBusService(options, transport)
    });

    afterEach(() => {
        jest.restoreAllMocks();
    });


    it("should init transport", async () => {
        const initTransport = jest.spyOn(transport, "initialize");
        initTransport.mockResolvedValue();
        await service.initialize();
        expect(initTransport).toHaveBeenCalledTimes(1);
    });

    it("should subscribe to topics if provided during init", async () => {
        const subscribe = jest.spyOn(transport, "subscribe");
        subscribe.mockReturnValue(Promise.resolve());
        await service.initialize();
        expect(subscribe).toHaveBeenCalledTimes(2)
    });

    it("should subscribe via transport", async () => {
        const subscribe = jest.spyOn(transport, "subscribe");
        subscribe.mockReturnValue(Promise.resolve());
        await service.subscribe("topic", (event: Event) => {return Promise.resolve()});
        expect(subscribe).toHaveBeenCalledWith("topic");
    });

    it("should relay messages as strings to the transport", async () => {
        const publish = jest.spyOn(transport, "publish");
        publish.mockResolvedValue();
        const message = { type: "add", value: "message", correlationId: "1234" }
        await service.publish("topic", message);
        expect(publish).toHaveBeenCalledWith("topic", message);
    });

    it("should use the provided logger", async () => {
        const logListener: SpyInstance = jest.spyOn(LoggerMock.prototype, "log");
        const logger = new LoggerMock();
        service = new KafkaEventBusService(options, transport, logger);
        await service.initialize();
        expect(logListener).toHaveBeenCalled();
        logListener.mockReset();
    });
});

