import EventBusOptions from "./dto/event-bus-options";
import { TransportProvider } from "./transport/transport-provider";
import { EventBusService, EventListener } from "./event-bus.service";
import { EventBusLogger } from "./logging/event-bus-logger";
import EventBusSubscribeError from "./error/event-bus-subscribe-error";
import EventBusConnectError from "./error/event-bus-connect-error";
import EventBusPublishError from "./error/event-bus-publish-error";
import EventBusDisconnectError from "./error/event-bus-disconnect-error";
import EventBusEventError from "./error/event-bus-event-error";
import { DefaultLogger } from "./logging/default-logger";
import Message from "./dto/message";

export class KafkaEventBusService implements EventBusService {
    private listeners: Map<string, EventListener[]>;

    constructor(
        private readonly options: EventBusOptions,
        private readonly transport: TransportProvider,
        private readonly logger: EventBusLogger = new DefaultLogger()
    ) {
        this.listeners = new Map<string, EventListener[]>();
        this.logger.setContext("KafkaEventBus");
    }

    async initialize(): Promise<void> {
        try {
            await this.transport.initialize();
            this.logger.log("transport initialized");
        } catch (e) {
            throw new EventBusConnectError("unable to connect: " + e.message);
        }

        if (this.options.topics !== undefined) {
            this.logger.log("subscribing to configured topics");
            this.subsribeToConfiguredTopics(this.options.topics);
        }
    }

    /**
     * Subscribes to topics supplied in the configurations
     * @param topics topics to subscribe to
     */
    async subsribeToConfiguredTopics(topics: string[]) {
        topics.forEach(async topic => {
            await this.subscribePreemptively(topic);
        });
    }

    /**
     * Requests the broker to listen to a topic without any given listeners. This allows new listeners to be added later
     * @param topicName the topic to listen to
     * @throws {EventBusSubscribeError} notably in events such as when subscribe is called on a never before subscribed topic, after calling listen()
     */
    async subscribePreemptively(topicName: string) {
        // If we haven't got any registered listeners for this topic, register them and subscribe
        try {
            if (!this.listeners.has(topicName)) {
                await this.transport.subscribe(topicName);
                this.listeners.set(topicName, []);
                this.logger.log(`subscribed to topic [${topicName}]`);
            }
        } catch (e) {
            throw new EventBusSubscribeError("unable to subscribe: " + e.message)
        }
    }

    /**
     * Begins listening for incoming events across the bus, call before using the event bus to send and recieve messages
     * @throws {EventBusEventError} in the event configuration 
     */
    async listen(): Promise<void> {
        await this.configureMessageConsumption();
    }

    /**
     * Sets up how the messages consumed by the consumer should be processed.
     * @throws {EventBusEventError} in case the configuration fails
     */
    private async configureMessageConsumption() {
        // Configure how we manage the messages we get
        try {
            await this.transport.onMessage(async (event) => {
                // Listeners may or may not have been registered even if the topic is subscribed to
                const eventListener = this.listeners.get(event.topic);
                if (eventListener && eventListener.length > 0) {
                    eventListener.forEach(listener => {
                        listener(event).catch(error => {
                            this.logger.error(`error occurred in listener for topic: ${event.topic}, error: ${JSON.stringify(error)}`)
                        });
                    });
                } else {
                    this.logger.error(`received message from topic: [${event.topic}], but no listeners`);
                }
            })
        } catch (e) {
            throw new EventBusEventError("error while configuring message handling: " + e.message);
        }
        this.logger.log('configured message consumer');
    }

    /**
     * Subscribes to the given topic and adds an event listener to handle events for this topic
     * @param topicName the topic to subscribe to
     * @param eventListener the listener to handle incoming events
     * @throws {EventBusSubscribeError} most often in case this is called on new topics after calling .listen()
     */
    async subscribe(topicName: string, eventListener: EventListener): Promise<void> {
        await this.subscribePreemptively(topicName);
        if (this.listeners.get(topicName)) {
            this.listeners.get(topicName).push(eventListener);
        }
    }

    /**
     * Publishes a message to the given topic
     * @param topicName the topic to publish the event to
     * @param message the message to publish
     * @throws {EventBusPublishError} in the event it is unable to publish
     */
    async publish(topicName: string, message: Message): Promise<void> {
        try {
            this.logger.log(`publishing message, correlationId: ${message.correlationId}`);
            await this.transport.publish(topicName, message)
        } catch (e) {
            throw new EventBusPublishError("unable to publish event: " + e.message);
        }
    }

    /**
     * Call before application shutdown
     * @throws {EventBusDisconnectError} in the event of failure to disconnect from the underlying transports
     */
    async destroy(): Promise<void> {
        try {
            await this.transport.destroy();
        } catch (e) {
            throw new EventBusDisconnectError("error while disconnecting from transport: " + e.message);
        }
    }
}

