import { TransportProvider } from "./transport-provider";
import EventBusOptions from "../dto/event-bus-options";
import { Kafka, Producer, Consumer, IHeaders, KafkaMessage, RecordMetadata, logLevel } from "kafkajs";
import Event from "../dto/event";
import { EventBusLogger } from "../logging/event-bus-logger";
import { DefaultLogger } from "../logging/default-logger";
import Message from "../dto/message";

export default class KafkaTransport implements TransportProvider {
    private kafka: Kafka;
    private producer: Producer;
    private consumer: Consumer;

    constructor(private readonly options: EventBusOptions, private logger: EventBusLogger = new DefaultLogger()) {
        this.logger.setContext("KafkaTransport");
        this.kafka = new Kafka({
            clientId: this.options.clientId,
            brokers: this.options.brokers,
            logCreator: () => this.logger.logCreator
        });
        this.producer = this.kafka.producer();
        this.consumer = this.kafka.consumer({ groupId: this.options.groupId });
    }

    async initialize(): Promise<void> {
        await this.producer.connect();
        await this.consumer.connect();
    }

    async subscribe(topicName: string): Promise<void> {
        await this.consumer.subscribe({ topic: topicName });
    }

    async publish(topicName: string, message: Message): Promise<void> {
        const metadata: RecordMetadata[] = await this.producer.send({
            topic: topicName,
            messages: [
                {
                    value: message.value,
                    headers: {
                        correlationId: Buffer.from(message.correlationId),
                        type: Buffer.from(message.type)
                    }
                },
            ],
        });

        const info = {
            correlationId: message.correlationId,
            metadata
        };

        this.logger.log(`message published: ${JSON.stringify(info)}`);
    }

    async onMessage(messageHandler: (payload: Event) => Promise<void>): Promise<void> {
        await this.consumer.run({
            eachMessage: async ({ topic, partition, message }) => {
                try {
                    const msg: Message = {
                        correlationId: this.tryGetHeader(message, "correlationId"),
                        type: this.tryGetHeader(message, "type"),
                        value: message.value.toString(),
                    }
                    this.logger.log(`message received, correlationId: ${msg.correlationId}`)
                    messageHandler({ topic, timestamp: message.timestamp.toString(), message: msg })
                        .then(() => {
                            this.logger.debug ? this.logger.debug(`message processed by handlers, correlationId: ${msg.correlationId}`) : undefined;
                        })
                        .catch((reason) => {
                            this.logger.error(`message handlers failed for message, correlationId: ${msg.correlationId}. error: ${JSON.stringify(reason)}`)
                        });
                } catch (e) {
                    this.logger.error(`invalid message received on topic ${topic}, partition: ${partition}, message: ${JSON.stringify(message)}, error: ${JSON.stringify(e)}`);
                }
            }
        })
    }

    private tryGetHeader(message: KafkaMessage, header: string) {
        if (message.headers && message.headers[header]) {
            return message.headers[header].toString();
        }
        return "";
    }

    async destroy(): Promise<void> {
        await this.producer.disconnect();
        await this.consumer.disconnect();
    }
}
